//--------------------------------------------------------------
// File     : mp45dt02_lolevel.c
// V : 1.0 / 15.06.2013 / UB
//--------------------------------------------------------------



//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "mp45dt02_lolevel.h"




//--------------------------------------------------------------
// Globale Variabeln
//--------------------------------------------------------------
PDMFilter_InitStruct Filter;




//--------------------------------------------------------------
// Interne Funktionen
//--------------------------------------------------------------
void P_MP45DT02LL_InitVar(void);
void P_MP45DT02LL_InitIO(void);
void P_MP45DT02LL_InitNVIC(void);
void P_MP45DT02LL_InitSPI(void);



//--------------------------------------------------------------
// stellt I2S-Clock auf 86MHz ein
//--------------------------------------------------------------
void UB_MP45DT02LL_SetI2SClock(void)
{
  RCC->CFGR &= ~RCC_CFGR_I2SSRC;

  /* Configure PLLI2S */
  RCC->PLLI2SCFGR = (MP45DT02_PLLI2S_N << 6) | (MP45DT02_PLLI2S_R << 28);

  /* Enable PLLI2S */
  RCC->CR |= ((uint32_t)RCC_CR_PLLI2SON);

  /* Wait till PLLI2S is ready */
  while((RCC->CR & RCC_CR_PLLI2SRDY) == 0)
  {
  }
}


//--------------------------------------------------------------
// init Funktion
//--------------------------------------------------------------
void UB_MP45DT02LL_Init(void)
{

  // Init der Variabeln
  P_MP45DT02LL_InitVar();

  // Enable CRC module
  RCC->AHB1ENR |= RCC_AHB1ENR_CRCEN;
    
  // Filter LP & HP Init
  Filter.LP_HZ = 8000;
  Filter.HP_HZ = 10;
  Filter.Fs = 16000;
  Filter.Out_MicChannels = 1;
  Filter.In_MicChannels = 1;
    
  PDM_Filter_Init((PDMFilter_InitStruct *)&Filter);  

  // init der GPIOs
  P_MP45DT02LL_InitIO();

  // init vom NVIC
  P_MP45DT02LL_InitNVIC();

  // init vom SPI
  P_MP45DT02LL_InitSPI();

}


//--------------------------------------------------------------
// startet die Aufnahme
//--------------------------------------------------------------
void UB_MP45DT02LL_RecordStart(void)
{

  // enable vom RX-Interrupt
  SPI_I2S_ITConfig(MP45DT02_I2S, SPI_I2S_IT_RXNE, ENABLE);
  // enable I2S
  I2S_Cmd(MP45DT02_I2S, ENABLE); 

}


//--------------------------------------------------------------
// Diese Funktion muss zyklisch aufgerufen werden
//
// Return_Wert : 0 = Puffer ist leer
//               1 = Puffer ist voll -> Pointer = WritePtr
//--------------------------------------------------------------
uint8_t UB_MP45DT02LL_RecordLoop(void)
{
  uint8_t ret_wert=0;

  // check ob Puffer der Audio-Daten voll
  if(MP45DT02_Puffer.raw_ovf>0) {
    // wenn Puffer voll
    MP45DT02_Puffer.raw_ovf=0;

    // Puffer umschalten
    if(MP45DT02_Puffer.akt_puffer==0) {      
      MP45DT02_Puffer.read_ptr=MP45DT02_Puffer.buf2_data;
      MP45DT02_Puffer.write_ptr=MP45DT02_Puffer.buf1_data;
      MP45DT02_Puffer.akt_puffer=1;
    }
    else {      
      MP45DT02_Puffer.read_ptr=MP45DT02_Puffer.buf1_data;
      MP45DT02_Puffer.write_ptr=MP45DT02_Puffer.buf2_data;
      MP45DT02_Puffer.akt_puffer=0;
    }    

    ret_wert=1;
  }

  return(ret_wert);
}


//--------------------------------------------------------------
// stoppt die Aufnahme
//--------------------------------------------------------------
void UB_MP45DT02LL_RecordStop(void)
{
  // I2S disable
  I2S_Cmd(MP45DT02_I2S, DISABLE);

  // Variabeln zuruecksetzen
  P_MP45DT02LL_InitVar();
}



//--------------------------------------------------------------
// interne Funktion
// init aller Variabeln
//--------------------------------------------------------------
void P_MP45DT02LL_InitVar(void)
{
  uint32_t n;

  // init der Variabeln
  for(n=0;n<MP45DT02_RAW_PUFFER_SIZE;n++) {
    MP45DT02_Puffer.raw_data[n]=0x00;
  }
  MP45DT02_Puffer.raw_cnt=0;
  MP45DT02_Puffer.raw_ovf=0;   
  for(n=0;n<MP45DT02_DATA_PUFFER_SIZE;n++) {
    MP45DT02_Puffer.buf1_data[n]=0x00;
    MP45DT02_Puffer.buf2_data[n]=0x00;
  }
  MP45DT02_Puffer.read_ptr =MP45DT02_Puffer.buf1_data;
  MP45DT02_Puffer.write_ptr=MP45DT02_Puffer.buf2_data;
  MP45DT02_Puffer.akt_puffer=0;
}


//--------------------------------------------------------------
// interne Funktion
// init der IO-Pins
//--------------------------------------------------------------
void P_MP45DT02LL_InitIO(void)
{
  GPIO_InitTypeDef GPIO_InitStructure;

  /* Enable GPIO clocks */
  RCC_AHB1PeriphClockCmd(MP45DT02_SCK_GPIO_CLK | MP45DT02_MOSI_GPIO_CLK, ENABLE);

  GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
  GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  GPIO_InitStructure.GPIO_PuPd  = GPIO_PuPd_NOPULL;
  GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

  /* SPI SCK pin configuration */
  GPIO_InitStructure.GPIO_Pin = MP45DT02_SCK_PIN;
  GPIO_Init(MP45DT02_SCK_GPIO_PORT, &GPIO_InitStructure);
  GPIO_PinAFConfig(MP45DT02_SCK_GPIO_PORT, MP45DT02_SCK_SOURCE, MP45DT02_GPIO_AF);
  
  /* SPI MOSI pin configuration */
  GPIO_InitStructure.GPIO_Pin =  MP45DT02_MOSI_PIN;
  GPIO_Init(MP45DT02_MOSI_GPIO_PORT, &GPIO_InitStructure);
  GPIO_PinAFConfig(MP45DT02_MOSI_GPIO_PORT, MP45DT02_MOSI_SOURCE, MP45DT02_GPIO_AF);
}


//--------------------------------------------------------------
// interne Funktion
// init vom Interrupt
//--------------------------------------------------------------
void P_MP45DT02LL_InitNVIC(void)
{
  NVIC_InitTypeDef NVIC_InitStructure;

  NVIC_PriorityGroupConfig(NVIC_PriorityGroup_3); 
  /* Configure the SPI interrupt priority */
  NVIC_InitStructure.NVIC_IRQChannel = MP45DT02_I2S_IRQ;
  NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 1;
  NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
  NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
  NVIC_Init(&NVIC_InitStructure);

}


//--------------------------------------------------------------
// interne Funktion
// init der I2S-Schnittstelle
//--------------------------------------------------------------
void P_MP45DT02LL_InitSPI(void)
{
  I2S_InitTypeDef I2S_InitStructure;

  /* Enable the SPI clock */
  RCC_APB1PeriphClockCmd(MP45DT02_I2S_CLK,ENABLE);
  
  /* SPI configuration */
  SPI_I2S_DeInit(MP45DT02_I2S);
  I2S_InitStructure.I2S_AudioFreq = MP45DT02_I2S_FRQ;
  I2S_InitStructure.I2S_Standard = I2S_Standard_LSB;
  I2S_InitStructure.I2S_DataFormat = I2S_DataFormat_16b;
  I2S_InitStructure.I2S_CPOL = I2S_CPOL_High;
  I2S_InitStructure.I2S_Mode = I2S_Mode_MasterRx;
  I2S_InitStructure.I2S_MCLKOutput = I2S_MCLKOutput_Disable;
  /* Initialize the I2S peripheral with the structure above */
  I2S_Init(MP45DT02_I2S, &I2S_InitStructure);

  /* Enable the Rx buffer not empty interrupt */
  SPI_I2S_ITConfig(MP45DT02_I2S, SPI_I2S_IT_RXNE, ENABLE); 
}


//--------------------------------------------------------------
// ISR-Funktion
// wird aufgerufen, wenn Daten vom MP45DT02 empfangen wurden
//
// T=15,6us => f=64kHz (fr ein 16bit Wert vom Micro)
// T=1ms    => f=1kHz  (fr 16x 16bit Werte [gefiltert])
// das ergibt => 62,5us pro 16bit Frame => 16kHz Samplefrequenz
//--------------------------------------------------------------
void MP45DT02_SPI_IRQHANDLER(void)
{     
  uint16_t akt_data;

  // check ob Audio-Daten vorhanden sind
  if (SPI_GetITStatus(MP45DT02_I2S, SPI_I2S_IT_RXNE) != RESET)
  {
    // Audio-Daten einlesen
    akt_data = SPI_I2S_ReceiveData(MP45DT02_I2S);
    // Audio-Daten speichern
    MP45DT02_Puffer.raw_data[MP45DT02_Puffer.raw_cnt] = HTONS(akt_data);
    MP45DT02_Puffer.raw_cnt++;
    
    // Check ob RAW-Array der Audio-Daten voll
    if (MP45DT02_Puffer.raw_cnt >= MP45DT02_RAW_PUFFER_SIZE)
    {
      MP45DT02_Puffer.raw_cnt = 0;  
         
      // Audio-Daten filtern und in Read-Puffer kopieren
      PDM_Filter_64_LSB((uint8_t *)MP45DT02_Puffer.raw_data, (uint16_t *)MP45DT02_Puffer.read_ptr, MP45DT02_RAW_VOLUME , (PDMFilter_InitStruct *)&Filter);

      // merker setzen, das Puffer voll ist
      MP45DT02_Puffer.raw_ovf = 1;       
    }
  }
}
