//--------------------------------------------------------------
// File     : mp45dt02_lolevel.h
//--------------------------------------------------------------


#ifndef __STM32F4_UB_MP45DT02_LOLEVEL_H
#define __STM32F4_UB_MP45DT02_LOLEVEL_H

//--------------------------------------------------------------
// includes
//--------------------------------------------------------------
#include "stm32f4xx.h"
#include "stm32f4xx_gpio.h"
#include "stm32f4xx_rcc.h"
#include "stm32f4xx_spi.h"
#include "misc.h"



//--------------------------------------------------------------
// I2S-Clock Defines :
// HSE-Clock = 8 MHz, PLL_M = 8
// I2S_VCO = (HSE / PLL_M) * PLLI2S_N => 258 MHz
// I2S_CLK = I2S_VCO / PLLI2S_R => 86 MHz
//--------------------------------------------------------------
#define MP45DT02_PLLI2S_N   258
#define MP45DT02_PLLI2S_R   3



//--------------------------------------------------------------
// I2S-Verbindung vom MP45DT02
// I2S per SPI2
// [SCK = PB10, SD = PC3]
//-------------------------------------------------------------- 
#define MP45DT02_SCK_PIN                 GPIO_Pin_10
#define MP45DT02_SCK_GPIO_PORT           GPIOB
#define MP45DT02_SCK_GPIO_CLK            RCC_AHB1Periph_GPIOB
#define MP45DT02_SCK_SOURCE              GPIO_PinSource10

#define MP45DT02_MOSI_PIN                GPIO_Pin_3
#define MP45DT02_MOSI_GPIO_PORT          GPIOC
#define MP45DT02_MOSI_GPIO_CLK           RCC_AHB1Periph_GPIOC
#define MP45DT02_MOSI_SOURCE             GPIO_PinSource3

#define MP45DT02_I2S                     SPI2
#define MP45DT02_I2S_CLK                 RCC_APB1Periph_SPI2
#define MP45DT02_GPIO_AF                 GPIO_AF_SPI2
#define MP45DT02_I2S_IRQ                 SPI2_IRQn
#define MP45DT02_SPI_IRQHANDLER          SPI2_IRQHandler


//--------------------------------------------------------------
#define MP45DT02_I2S_FRQ                 32000

//--------------------------------------------------------------
#define MP45DT02_SAMPL_FRQ               16000


//--------------------------------------------------------------
#define MP45DT02_RAW_PUFFER_SIZE         64
#define MP45DT02_DATA_PUFFER_SIZE        16

#define MP45DT02_RAW_VOLUME              50


//--------------------------------------------------------------
// interne Struktur vom Puffer
//--------------------------------------------------------------
typedef struct {
  uint16_t raw_data[MP45DT02_RAW_PUFFER_SIZE]; 
  uint32_t raw_cnt;
  uint8_t  raw_ovf;  
  uint16_t buf1_data[MP45DT02_DATA_PUFFER_SIZE];
  uint16_t buf2_data[MP45DT02_DATA_PUFFER_SIZE];
  uint16_t* read_ptr;
  uint16_t* write_ptr;
  uint8_t  akt_puffer;
}MP45DT02_Puffer_t;
MP45DT02_Puffer_t MP45DT02_Puffer;


//--------------------------------------------------------------
// interne Struktur vom Filter
//--------------------------------------------------------------
typedef struct {
  uint16_t Fs;
  float LP_HZ;
  float HP_HZ;
  uint16_t In_MicChannels;
  uint16_t Out_MicChannels;
  char InternalFilter[34];
}PDMFilter_InitStruct;


//--------------------------------------------------------------
// Makro
//--------------------------------------------------------------
#define HTONS(A)  ((((uint16_t)(A) & 0xff00) >> 8) | \
                   (((uint16_t)(A) & 0x00ff) << 8))



//--------------------------------------------------------------
// Funktionen der Filter Library : "libPDMFilter_GCC.a"
//--------------------------------------------------------------
void PDM_Filter_Init(PDMFilter_InitStruct * Filter);
int32_t PDM_Filter_64_MSB(uint8_t* data, uint16_t* dataOut, uint16_t MicGain,  PDMFilter_InitStruct * Filter);
int32_t PDM_Filter_80_MSB(uint8_t* data, uint16_t* dataOut, uint16_t MicGain,  PDMFilter_InitStruct * Filter);
int32_t PDM_Filter_64_LSB(uint8_t* data, uint16_t* dataOut, uint16_t MicGain,  PDMFilter_InitStruct * Filter);
int32_t PDM_Filter_80_LSB(uint8_t* data, uint16_t* dataOut, uint16_t MicGain,  PDMFilter_InitStruct * Filter);



//--------------------------------------------------------------
void UB_MP45DT02LL_SetI2SClock(void);
void UB_MP45DT02LL_Init(void);
void UB_MP45DT02LL_RecordStart(void);
uint8_t UB_MP45DT02LL_RecordLoop(void);
void UB_MP45DT02LL_RecordStop(void);





 
#endif // __STM32F4_UB_MP45DT02_LOLEVEL_H


